## 引用元
【Python/tkinter】[神経衰弱ゲームの作り方](https://daeudaeu.com/tkinter-concentration/)より引用

## 使用したimport
```python
import tkinter
import tkinter as tk
import random
from playsound import playsound
```

## カードのサイズ
```python
CARD_WIDTH = 150
CARD_HEIGHT = 210
```

## サウンド
```python
def func1():
    playsound("クイズ正解2.mp3") # 同じ記号の時の音

def func2():
    playsound("ビープ音4.mp3") # 違う記号の時の音
    
def func3():
    playsound("歓声と拍手.mp3") # クリアしたときの音
```

## 神経衰弱の中身
```python
class Concentration:
    def __init__(self, master):
        self.master = master
        
        # 選択中のカード
        self.first_card_id = None
        self.second_card_id = None
        
        # キャンバスのサイズ
        self.width = CARD_WIDTH*9 # 横に9枚並べるためのサイズ
        self.height = CARD_HEIGHT*4 # 縦に4枚並べるためのサイズ

        # 未獲得のカードを管理するリスト
        self.remain_card_ids = []

        self.createWidgets() # キャンバス作成
        self.createCards() # カードを作成
        self.layoutCards() # カードを並べる
        self.setEvents() # イベントの受付設定
```

### 変更点
カードの枚数を変更した
```python
    # ウェジェット作成
    def createWidgets(self):

        self.canvas = tkinter.Canvas(self.master,width=self.width,height=self.height,bg="white",highlightthickness=0)
        self.canvas.pack()

    # 神経衰弱で使用する記号を作る
    def createCards(self):

        numbers = ["A", "B", "C", "D", "E", "F", "G", "H", "I"]

        # 9種類4枚ずつカードを作成
        self.cards = [number for _ in range(4) for number in numbers]

        # カードの並びをシャッフルする
        random.shuffle(self.cards)

    # カードをばらばらに並べる
    def layoutCards(self):

        for i, number in enumerate(self.cards):

            # 水平方向と垂直方向の位置
            h = i % 9
            v = i // 9

            # カードを表現する長方形の座標を計算
            x1 = h * CARD_WIDTH
            x2 = (h + 1) * CARD_WIDTH
            y1 = v * CARD_HEIGHT
            y2 = (v + 1) * CARD_HEIGHT

            # カードの中心に数字を描画
            self.canvas.create_text(x1 + CARD_WIDTH / 2, y1 + CARD_HEIGHT / 2,text=number,font=("", 40))

            # カードの数字を隠す
            fig_id = self.canvas.create_rectangle(x1, y1, x2, y2,fill="lightgreen",tag=number)

            # 未獲得のカードとしてリストに追加
            self.remain_card_ids.append(fig_id)
```

### 変更点
カードの記号の数と枚数、水平方向と垂直方向の位置を変更

```python
    # 神経衰弱の機能制作
    def setEvents(self):

        # クリック時にfaceupCardが実行されるように設定
        self.canvas.bind("<ButtonPress>", self.selectCard)

    # 選択されたカードについて
    def selectCard(self, event):

        # ３枚同時に選択された場合は何もしない
        if self.first_card_id is not None and self.second_card_id is not None:
            return

        # クリックされたカードに対応する図形IDを取得
        card_fig_ids = self.canvas.find_closest(event.x, event.y)
        card_fig_id = card_fig_ids[0]

        # クリックされた図形が未獲得カードでない場合は何もしない
        if not card_fig_id in self.remain_card_ids:
            return

        # 同じ図形がクリックされた場合は何もしない
        if card_fig_id == self.first_card_id:
            return

        # 取得したIDの図形を塗りつぶし無しにする（数字が見えるようになる）
        self.canvas.itemconfig(card_fig_id, fill="")

        if self.first_card_id is None:

            # １枚目に選択したカードとして覚えておく
            self.first_card_id = card_fig_id
        else:

            # ２枚目に選択したカードとして覚えておく
            self.second_card_id = card_fig_id

            # 図形IDから表向きにされたカードの数字を取得する
            first_number = self.canvas.gettags(self.first_card_id)[0]
            second_number = self.canvas.gettags(self.second_card_id)[0]
```

```python
            if first_number == second_number:

                # 選んだカードが同じ数字だった場合
                self.earnCards()
            else:

                # 選んだカードが異なる数字だった場合
                self.canvas.unbind("") # 一時的にクリックを無効にする
                app.after(200,func2) # 不正解の音を鳴らす

                # 200ミリ秒後にカードを裏向きにする
                self.master.after(200, self.reverseCards)
```

### 変更点
選んだカードが異なる数字だった時不正解の音を鳴らす

```python
    # カードを戻す
    def reverseCards(self):

        # 数字が隠れる
        self.canvas.itemconfig(self.first_card_id, fill="lightgreen")
        self.canvas.itemconfig(self.second_card_id, fill="lightgreen")

        # 選択中のカードの図形IDをNoneに設定
        self.first_card_id = None
        self.second_card_id = None

        # 再度クリック時にselectCardが実行されるように設定
        self.canvas.bind("<ButtonPress>", self.selectCard)
```
### 変更点
カードの色を変えた

```python
    # めくったカードを獲得済みにする
    def earnCards(self):

        # 揃ったカードの色をグレーにする
        self.canvas.itemconfig(self.first_card_id, fill="gray")
        self.canvas.itemconfig(self.second_card_id, fill="gray")
        app.after(100,func1) # 正解の音を鳴らす
```
### 変更点
成功したとき正解の音を鳴らす
```python
        # 数字が見えるようになる
        self.canvas.lower(self.first_card_id, "all")
        self.canvas.lower(self.second_card_id, "all")
        
        # 未獲得カードのリストから獲得されたカードを削除する
        self.remain_card_ids.remove(self.first_card_id)
        self.remain_card_ids.remove(self.second_card_id)

        # 選択中のカードの図形IDをNoneに設定
        self.first_card_id = None
        self.second_card_id = None

        # 未獲得カードのリストに要素がなくなったらゲームクリア
        if len(self.remain_card_ids) == 0:
            app.after(500,func3),self.canvas.create_text(self.width / 2, self.height / 2,font=("", 40),text="Congratulation!!!",fill="lightpink")
```
### 変更点
クリア後のテキストと文字の色を変更した

## ウィジェット表示
```python
app = tkinter.Tk()
app.title('神経衰弱') # 題名
app.iconphoto(False, tk.PhotoImage(file='board_game.png')) # アイコン
Concentration(app)
app.mainloop()
```